@@ -166,6 +166,7 @@ static int animating = 0;
 static int unoptimizing = 0;
 static int install_colormap = 0;
 static int interactive = 1;
+static int min_delay = 0;
 
 static struct timeval preparation_time;
 
@@ -183,6 +184,7 @@ static struct timeval preparation_time;
 #define BACKGROUND_OPT		310
 #define NEW_WINDOW_OPT		311
 #define TITLE_OPT		312
+#define MIN_DELAY_OPT		313
 
 #define WINDOW_TYPE		(Clp_ValFirstUser)
 
@@ -195,6 +197,7 @@ const Clp_Option options[] = {
   { "install-colormap", 'i', INSTALL_COLORMAP_OPT, 0, Clp_Negate },
   { "interactive", 'e', INTERACTIVE_OPT, 0, Clp_Negate },
   { "help", 0, HELP_OPT, 0, 0 },
+  { "min-delay", 0, MIN_DELAY_OPT, Clp_ValInt, Clp_Negate },
   { "name", 0, NAME_OPT, Clp_ValString, 0 },
   { "title", 'T', TITLE_OPT, Clp_ValString, 0 },
   { "unoptimize", 'U', UNOPTIMIZE_OPT, 0, Clp_Negate },
@@ -267,6 +270,7 @@ Options are:\n\
       --new-window WINDOW       Show GIF in new child of existing WINDOW.\n\
   -i, --install-colormap        Use a private colormap.\n\
   --bg, --background COLOR      Use COLOR for transparent pixels.\n\
+      --min-delay DELAY         Set minimum frame delay to DELAY/100 sec.\n\
   +e, --no-interactive          Ignore buttons and keystrokes.\n\
       --help                    Print this message and exit.\n\
       --version                 Print version number and exit.\n\
@@ -627,6 +631,8 @@ schedule_next_frame(Gt_Viewer *viewer)
   struct timeval interval;
   int delay = viewer->im[viewer->im_pos]->delay;
   int next_pos = viewer->im_pos + 1;
+  if (delay < min_delay)
+    delay = min_delay;
   if (next_pos == viewer->nim)
     next_pos = 0;
 
@@ -1303,6 +1309,10 @@ main(int argc, char *argv[])
       interactive = clp->negated ? 0 : 1;
       break;
 
+    case MIN_DELAY_OPT:
+      min_delay = clp->negated ? 0 : clp->val.i;
+      break;
+
      case VERSION_OPT:
       printf("gifview (LCDF Gifsicle) %s\n", VERSION);
       printf("Copyright (C) 1997-2011 Eddie Kohler\n\
